;-------------------------------------------------------------------------------------
; video_postprocess_boomerang.ahk - an AutoHotKey script to postprocess a video after
;	capture in DSLR Remote Pro to create a boomerang video
;-------------------------------------------------------------------------------------
; This script can rotate, resize and add an overlay and soundtrack to a video
; captured using DSLR Remote Pro v3.11.3 onwards. It uses ffmpeg to modify the
; video.
;
; Usage: set the "Delay before playback" to 30 seconds in the video booth
; settings in DSLR Remote Pro and then double click on this script to run it.
;
; Command line usage to process a single video by reading the XML summary file:
; Open a command prompt window, cd to the folder containing this
; script and then run ot with the pathname of the video booth XML summary file
; e.g. video_postprocess.ahk C:\Users\chris\Documents\PhotoboothImages\2018-10-31\IMG_0001.xml
;
; Add an optional overlay by placing a PNG file named video_overlay.png in the photo booth images folder
; Add an optional soundtrack by placing an audio file named video_soundtrack.mp3 or video_soundtrack.wav
; in the photo booth images folder
; Add an optional 'intro' video at the start by placing a video file named video_intro.mp4 in the photo booth images folder
; Add an optional 'outro' video to the end by placing a video file named video_outro.mp4 in the photo booth images folder
; IMPORTANT: the video_intro.mp4 and video_outro.mp4 files must be the same size as the output video otherwise the script won't work
;
; Edit this script to change the following variables:
; copyFolder - the name of the subfolder in which to save a copy of the original video
; boomerang - set this to 1 to create forwards/backwards boomerang or set to 0 to create forwards only video
; rotateDegreesClockwise - optional rotation in 90 degree intervals
; videoHeight - height of the video (after rotation). Set this to 0 to disable resizing
; videoPreset - ffmpeg preset for video CODEC. Set to veryslow, slow, fast, veryfast or ultrafast
; cropWidth - optional cropping of video in pixels
; loops - the number of times to loop the boomerang video
; playbackSpeed - speed up or slow down the video
;
; FFmpeg can be downloaded from https://www.ffmpeg.org/
;
; AutoHotKey is free and can be downloaded from:
; http://www.autohotkey.com
;
; This script comes with no warranty or support whatsoever and may be freely
; copied or modified as required.
;
; Written by Chris Breeze, www.breezesys.com
; 20181112: Initial version based on the original video_postprocess.ahk script
; 20181126: Added optional video_intro.mp4
; 20190405: Added bomerang option
;----------------------------------------------------------------------------------------
#singleinstance force
SetTitleMatchMode,2
StringCaseSense, Off

; subfolder in which to save a copy of the original video
copyFolder := "originals"

; boomerang: set this to 1 to play the video forwards then backwards or 0 to play forwards only
boomerang := 1

; rotate video in 90 degree increments, valid values: 0, 90, 180, 270
rotateDegreesClockwise := 0

; height of the video (after rotation). Set this to 0 to disable resizing
videoHeight := 600

; Crop the video to this width (after rotation). Set this to 0 to disable cropping
cropWidth := 0

; the number of times to loop the video
loops := 3

; video playback speed
playbackSpeed := 2

; ffmpeg video preset, choose from: veryslow, slow, fast, veryfast and ultrafast
videoPreset := "veryfast"

; set this to 1 to display debug info (0 to disable debug)
showDebugMessages := 0

;----------- Please don't change anything below here unless you know what you're doing! ----------------

; make sure videoHeight is an even number
if Mod(videoHeight, 2)
{
	videoHeight++
}

ffmpeg = %A_ScriptDir%\ffmpeg.exe
ifNotExist,%ffmpeg%
{
	MsgBox ERROR: ffmpeg.exe not found.`nPlease make sure ffmpeg.exe is in the same folder as this script:`n%A_ScriptDir%
	exitapp
}

if showDebugMessages
{
	; display a debug window at the top of the screen
	Gui +LastFound +AlwaysOnTop +ToolWindow  ; +ToolWindow avoids a taskbar button and an alt-tab menu item.
	Gui, Color, cBlack
	Gui, Font, s12 
	Gui, Add, Text, vStatusText cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Add, Text, vStatusText2 cLime, AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	Gui, Show,x100 y0 NoActivate,%A_ScriptName%  ; NoActivate avoids deactivating the currently active window.
	GuiControl,,StatusText,Please wait...
	GuiControl,,StatusText2,
}

; look for XML file in the command line
if %0%
{
	path = %1%
	SplitPath, path,,, ext
	if (ext = "mp4")
	{
		video = %1%
		screenDir := "."
	}
	else if (ext = "xml")
	{
		IfNotExist, %1%
		{
			MsgBox ERROR: XML file not found: %1% 
			exitapp
		}
		; read XML file, process video and exit
		FileRead, Contents, %1%
		RegExMatch(Contents, "<video_information>", video) 
		RegExMatch(Contents, "<photobooth_images_folder>(.*)</photobooth_images_folder>", photoboothFolder) 
		RegExMatch(Contents, "<download_folder>(.*)</download_folder>", downloadFolder) 
		RegExMatch(Contents, "<video_file>(.*)</video_file>", file) 

		if not Contents contains <video_information>
		{
			MsgBox ERROR: not a video booth summary file: %1%
			exitapp
		}
		if photoboothFolder < 1
		{
			MsgBox ERROR: video booth summary file doesn't contain <photobooth_images_folder> tag
			exitapp
		}
		if downloadFolder < 1
		{
			MsgBox ERROR: video booth summary file doesn't contain <download_folder> tag
			exitapp
		}
		if file < 1
		{
			MsgBox ERROR: video booth summary file doesn't contain <video_file> tag
			exitapp
		}

		screenDir = %photoboothFolder1%
		video = %downloadFolder1%\%file1%
	}
	else
	{
		MsgBox XML or MP4 image not found 
		exitapp
	}

	IfNotExist, %video%
	{
		MsgBox ERROR: video file not found: %video% 
		exitapp
	}
	IfNotExist, %screenDir%
	{
		;MsgBox ERROR: Screen folder not found: %screenDir% 
		;exitapp
	}
	showDebugMessages = 1
	ProcessVideo(video, screenDir)
	exitapp
}

; loop waiting for video files to process
DebugMessage(1,"Window title")
Loop
{
	IfWinExist, Breeze Systems Photobooth - 
	{
		WinGetTitle, Title, Breeze Systems Photobooth - 
		if title contains video_processing
		{
			RegExMatch(title, "\s-\s(.*)", screen) 
			SplitPath, screen1, screenFilename, screenDir
			DebugMessage(1, "Screen folder: " screenDir)
			ControlGetText, msg, Static2, Breeze Systems Photobooth
			RegExMatch(msg, ",\s*movie=(.*)", video) 
			IfExist, %video1%
			{
				ProcessVideo(video1, screenDir)

				; tell photo booth the processed video is ready
				WinActivate, Breeze Systems Photobooth -
				Send ^z
				DebugMessage(2, "Sending Ctrl+Z to photo booth...")
				Sleep 1000
			}
			else
			{
				DebugMessage(2, "Waiting for movie to download...")
			}
		}
		else
		{
			DebugMessage(1, Title)
			DebugMessage(2, "Waiting for video_processing screen...")
		}
	}
	else
	{
		IfWinExist, DSLR Remote Pro for Windows -
		{
			DebugMessage(1, "Not running in photo booth mode")
		}
		else
		{
			DebugMessage(1, "DSLR Remote Pro is not running")
		}
		DebugMessage(2, "")
	}
	Sleep 100	; sleep for 1/10th sec to avoid hogging the processor
}

ProcessVideo(video, screenFolder)
{
	global ffmpeg, copyFolder, videoHeight, rotateDegreesClockwise
	global videoPreset, showDebugMessages
	global boomerang, playbackSpeed, cropWidth, loops

	DebugMessage(2, "Processing " video)

	rotate :=
	if rotateDegreesClockwise = 90
	{
		rotate := "transpose=1"
	}
	else if rotateDegreesClockwise = 180
	{
		rotate := "transpose=2,transpose=2"
	}
	else if rotateDegreesClockwise = 270
	{
		rotate := "transpose=2"
	}

	overlay = %screenFolder%\video_overlay.png
	intro = %screenFolder%\video_intro.mp4
	outro = %screenFolder%\video_outro.mp4

	audio = %screenFolder%\video_soundtrack.mp3
	IfNotExist,%audio%
	{
		audio = %screenFolder%\video_soundtrack.wav
	}

	; make a copy of the original video and delete any temp files
	SplitPath,video,name,folder
	videoCopyFolder = %folder%\%copyFolder%
	FileCreateDir,%videoCopyFolder%
	videoCopy = %videoCopyFolder%\%name%
	videoTemp = %videoCopyFolder%\temp_%name%
	videoTempReversed = %videoCopyFolder%\temp_reversed_%name%
	videoProcessed = %videoCopyFolder%\processed_%name%
	FileDelete,%videoCopy%
	FileDelete,%videoTemp%
	FileDelete,%videoTempReversed%
	FileDelete,%videoProcessed%
	FileCopy,%video%,%videoCopy%

	; First pass: speed up, crop, rotate, scale, add overlay and drop audio
	inputFiles = -i "%video%"
	filter :=
	if playbackSpeed <> 1
	{
		speed := 1 / playbackSpeed
		filter = setpts=%speed%*PTS
	}
	if cropWidth
	{
		if filter
		{
			filter = %filter%,crop=%cropWidth%
		}
		else
		{
			filter = crop=%cropWidth%
		}
	}
	if rotate
	{
		if filter
		{
			filter = %filter%,%rotate%
		}
		else
		{
			filter = %rotate%
		}
	}
	if videoHeight > 0
	{
		if filter
		{
			filter = %filter%,scale=-2:%videoHeight%
		}
		else
		{
			filter = scale=-2:%videoHeight%
		}
	}
	if filter
	{
		filter = [0:v]%filter%
	}
	else
	{
		filter = [0:v]copy
	}
	IfExist,%overlay%
	{
		inputFiles = %inputFiles% -i "%overlay%"
		filter = %filter%[bg],[bg][1:v]overlay=0:0
	}
	cmd = "%ffmpeg%" -y %inputFiles% -filter_complex "%filter%" -c:v libx264 -preset %videoPreset% -an "%videoTemp%"
	runwait,%cmd%,,Hide
	; FileAppend,%cmd%,test.bat
	; MsgBox 1) %cmd%

	if boomerang
	{
		; Second pass: create reversed copy
		cmd = "%ffmpeg%" -y -i "%videoTemp%" -vf reverse "%videoTempReversed%"
		runwait,%cmd%,,Hide
		; FileAppend,%cmd%,test.bat
		; MsgBox 2) %cmd%
	}

	; Third pass: join them together with optional intro and outro videos
	if loops < 1
	{
		loops := 1
	}
	numInputs := 0
	inputFiles :=

	filter :=
	IfExist,%intro%
	{
		inputFiles = %inputFiles% -i "%intro%"
		filter = %filter%[%numInputs%:v:0]
		numInputs := numInputs + 1
	}
	Loop, %loops%
	{
		inputFiles = %inputFiles% -i "%videoTemp%"
		filter = %filter%[%numInputs%:v:0]
		numInputs := numInputs + 1
		if boomerang
		{
			inputFiles = %inputFiles% -i "%videoTempReversed%"
			filter = %filter%[%numInputs%:v:0]
			numInputs := numInputs + 1
		}
	}
	IfExist,%outro%
	{
		inputFiles = %inputFiles% -i "%outro%"
		filter = %filter%[%numInputs%:v:0]
		numInputs := numInputs + 1
	}
	IfExist,%audio%
	{
		inputFiles = %inputFiles% -i "%audio%"
		filter = %filter% concat=n=%numInputs%:v=1:a=0 [v]
		mapAndCodec = -map "[v]" -map %numInputs%:a -c:v libx264 -preset %videoPreset% -shortest
	}
	else
	{
		filter = %filter% concat=n=%numInputs%:v=1 [v]
		mapAndCodec = -map "[v]" -preset %videoPreset% -c:v libx264 
	}
	cmd = "%ffmpeg%" -y %inputFiles% -filter_complex "%filter%" %mapAndCodec% "%videoProcessed%"
	runwait,%cmd%,,Hide
	; FileAppend,%cmd%,test.bat
	; MsgBox 3) %cmd%

	; delete temporary files and overwrite original with processed video
	FileDelete, %videoTemp%
	FileDelete,%videoTempReversed%
	FileMove,%videoProcessed%,%video%,1
}

; Display a debug message if showDebugMessages = 1
; num specifies the line on which to display the message (1 or 2)
; msg is the message to display
DebugMessage(num, msg)
{
	global cStatusText, cStatusText2
	global showDebugMessages
	msg := A_Hour ":" A_Min ":" A_Sec ": " msg
	if showDebugMessages
	{
		if num=1
		{
			GuiControl,,StatusText,%msg%
		}
		else
		{
			GuiControl,,StatusText2,%msg%
		}
	}
}

GuiClose:
ExitApp
